#include "gaim-assistant.h"
#include "gaim-assistant-ui.h"
#include "gaim-assistant-util.h"

 


static GSList *last_auto_responses = NULL;
struct last_auto_response {
  GaimConnection *gc;
  char name[80];
  time_t sent;
};
static gboolean plugin_load(GaimPlugin *plugin);
static gboolean plugin_unload(GaimPlugin *plugin);
static GList * actions(GaimPlugin *plugin, gpointer context);
static GaimGtkPluginUiInfo ui_info =
{
	get_config_frame
};
static GaimPluginInfo info =
{
#ifdef GAIM_PLUGIN_API_VERSION
	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
#else
  GAIM_PLUGIN_MAGIC,
  1,
  0,
#endif
	GAIM_PLUGIN_STANDARD,                             /**< type           */
	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
	0,                                                /**< flags          */
	NULL,                                             /**< dependencies   */
	GAIM_PRIORITY_DEFAULT,                            /**< priority       */

	GASST_PLUGIN_ID,                                  /**< id             */
	N_("Gaim Assistant"),                             /**< name           */
	N_(GASST_VERSION),                                          /**< version        */
	                                                  /**  summary        */
	N_("Provides a variety of things to have your account do for you."),
	                                                  /**  description    */
	N_("Provides a variety of things to have your account do for you."),
	GASST_EMAIL,                                      /**< author         */
	GASST_SITE,                   /**< homepage       */

	plugin_load,                                      /**< load           */
	plugin_unload,                                    /**< unload         */
	NULL,                                             /**< destroy        */

	&ui_info,                                         /**< ui_info        */
	NULL,                                              /**< extra_info     */
	NULL,
	actions
};


void *im_recv_im(GaimAccount *account, char *sender, char *message, int flags);
static gboolean chat_recv_im(GaimAccount *account, char *sender, char *message, GaimConversation *conv);
static void init_plugin(GaimPlugin *plugin);
struct last_auto_response *get_last_auto_response(GaimConnection *gc, const char *name);
gboolean expire_last_auto_responses(gpointer data);
void flush_last_auto_responses(GaimConnection *gc);





void *im_recv_im(GaimAccount *account, char *sender, char *message,
           int flags)
{
  gaim_debug(GAIM_DEBUG_INFO, "gaim-assistat",
      "Flags in im_recv_im: %i\n", flags);
  if(flags ){
    gaim_debug(GAIM_DEBUG_INFO, "gaim-assistant",
        "Auto-response detected.  Not forwarding.\n");
    return NULL;
  }
    
  const char *origin = gaim_prefs_get_string("/plugins/gtk/X11/gassist/origin_account");
  GaimAccount *origin_account = gaim_accounts_find(origin, NULL);
  gaim_debug(GAIM_DEBUG_INFO, "gaim-assistant", "entered im_recv_im\n");
  GaimConnection *orig_acct_conn = NULL;
  if(origin!=NULL){
	  if(!strncmp(origin, "NULL", 4)){
		// set the origion connection to the incoming account
		orig_acct_conn = gaim_account_get_connection(account);
		origin_account=account;
	  }
	  else{
	  	// see if the account mentioned in origin is signed on
	  	if(gaim_account_is_connected(origin_account)){
	  		orig_acct_conn = gaim_account_get_connection(origin_account);
	  	}
	  	else{
	  		orig_acct_conn = gaim_account_get_connection(account);
	  		origin_account = account;
	  	}
	  }
  }
  else{
  	orig_acct_conn = gaim_account_get_connection(account);
  	origin_account=account;
  }
			  	
  // get the connection for this account
  GaimConnection *acct_conn = gaim_account_get_connection(account);
  
  // inserting a check here for away, away could be signified by
  // acct_conn->away==NULL or acct_conn->away=="" check for both 
  // and set a boolean bit.
  int iamaway=0;
  if(acct_conn->away!=NULL){
    if(acct_conn->away[0]!='\0')
      iamaway=1;
  }
      
  
  // if forward_msgs is enabled, and the person is away, then off we go...
  if(gaim_prefs_get_bool("/plugins/gtk/X11/gassist/forward_msgs")==TRUE  &&
      iamaway ){

    // get the conversation of the sender.
  	GaimConversation *conv = gaim_find_conversation_with_account(sender, account);
    
    // get the IM session of the sender
	  GaimConvIm *im = gaim_conversation_get_im_data(conv);


    // first thing's first, see if the sender is
    // on our buddy list or not. But only do this if the user set that option.
    GaimBuddy *sender_buddy = gaim_find_buddy(account, sender);
    if(sender_buddy==NULL && gaim_prefs_get_bool("/plugins/gtk/X11/gassist/blist_only")==TRUE){
      gaim_debug(GAIM_DEBUG_INFO, "gaim-assistant", "buddy %s not on buddy list and blist_only set.  Not forwarding.\n",
          sender);
      return FALSE;
    }

    // set up the variable to send the destination screen name back to the sender
    // if it is so configured.
    char sec_screenname[256];
    if(gaim_prefs_get_bool("/plugins/gtk/X11/gassist/show_fwdscrnm")==TRUE)
      if(gaim_prefs_get_bool("/plugins/gtk/X11/gassist/show_fwdscrnm_blist_only")==TRUE)
        if(sender_buddy==NULL)
          sec_screenname[0]='\0';
        else
          g_snprintf(sec_screenname, sizeof(sec_screenname), ", %s,", gaim_prefs_get_string("/plugins/gtk/X11/gassist/fwd_scrnm"));
      else
        g_snprintf(sec_screenname, sizeof(sec_screenname), ", %s,", gaim_prefs_get_string("/plugins/gtk/X11/gassist/fwd_scrnm"));
    else
      sec_screenname[0]='\0';
    
    
    // find the buddy spec'ed in fwd_scrnm and see if they are online.
    // if they don't exist on the account auto add it.
    GaimBuddy *buddy = gaim_find_buddy(origin_account, gaim_prefs_get_string("/plugins/gtk/X11/gassist/fwd_scrnm"));
    if(buddy==NULL && gaim_prefs_get_bool("/plugins/gtk/X11/gassist/online_check")){
      // didn't find the buddy on this account, we will auto add it. :-)
      buddy=gaim_buddy_new(origin_account, gaim_prefs_get_string("/plugins/gtk/X11/gassist/fwd_scrnm"), NULL);
      if(buddy==NULL && gaim_prefs_get_bool("/plugins/gtk/X11/gassist/online_check")){
        gaim_prefs_set_bool("/plugins/gtk/X11/gassist/forward_msgs", FALSE);
        return FALSE;
      }
      gaim_blist_add_buddy(buddy, NULL, NULL, NULL);
      serv_add_buddy(orig_acct_conn, buddy);

    }

	int send_anyway=0;
	if(buddy==NULL && gaim_prefs_get_bool("/plugins/gtk/X11/gassist/online_check")==FALSE)
		send_anyway=1;
	else{
		if(buddy->present>0)
			send_anyway=1;
	}


    if(send_anyway){
	  gaim_debug(GAIM_DEBUG_INFO, "gaim-assistant", "sending message to %s\n",
        gaim_prefs_get_string("/plugins/gtk/X11/gassist/fwd_scrnm"));
      if(orig_acct_conn==NULL){
      	printf("gaim-assistant: OMFG orig_acct_conn is NULL in im_recv_im!!!! RUNNNN!!!!\n");
      }
      else{
	      // the buddy is online.
    	  // or we are ignoring online status
	      // now send them an IM
	      char buff[1024];
    	  g_snprintf(buff, sizeof(buff), "User: %s sent %s Message:  %s",
    	    sender, account->username, message);

	      // send the buddy an IM marked as an AUTO-RESPONSE letting them know what's going on
    	  serv_send_im(orig_acct_conn, gaim_prefs_get_string("/plugins/gtk/X11/gassist/fwd_scrnm"), 
        	buff, 0);
      }


    }
    else{
      // the buddy is offline.  
      
      // queue the message into the message queue if the option to do 
      // so is set.  Otherwise, do nothing.
      if(gaim_prefs_get_bool("/plugins/gtk/X11/gassist/que_fwd_msgs")==TRUE){
        struct queued_message *qm;
        qm = g_new0(struct queued_message, 1);
	  		g_snprintf(qm->name, sizeof(qm->name), "%s", sender);
		  	qm->message = g_strdup(message);
			  qm->account = origin_account;
  			qm->tm = time(NULL);
	  		qm->flags = (GaimMessageFlags)flags;
		  	gasst_queue = g_slist_append(gasst_queue, qm);
      
        
      }

    }
   
    struct last_auto_response *lar = get_last_auto_response(acct_conn, sender);
    if (((time(NULL) - lar->sent) > 
        strtol(gaim_prefs_get_string("/plugins/gtk/X11/gassist/auto_resend_timeout"), NULL, 0)) && 
        gaim_prefs_get_bool("/plugins/gtk/X11/gassist/auto_reply")) {

      char buff[1024];

      g_snprintf(buff, sizeof(buff), USER_RESPONSE_STR, GASST_SITE,
      account->username, sec_screenname);
          
      // now send a reply back to the person who sent the original message letting them know what is going on.
      gaim_conv_im_write(im, account->username, buff, GAIM_MESSAGE_AUTO_RESP | GAIM_MESSAGE_SEND , time(NULL));
      serv_send_im(acct_conn, sender, buff, GAIM_MESSAGE_AUTO_RESP | GAIM_MESSAGE_SEND);

      // need to also send the away message so they get that too.  
      serv_send_im(acct_conn, sender, acct_conn->away, GAIM_MESSAGE_AUTO_RESP | GAIM_MESSAGE_SEND);    

  }


  }
    
	

	return FALSE;
}
/* --- NOTICE: the following is ripped from server.c cuz it works and is not set as extern. ---*/

                                                                                                                                                              
gboolean expire_last_auto_responses(gpointer data)
{
  GSList *tmp, *cur;
  struct last_auto_response *lar;
                                                                                                                                                              
  tmp = last_auto_responses;
                                                                                                                                                              
  while (tmp) {
    cur = tmp;
    tmp = tmp->next;
    lar = (struct last_auto_response *)cur->data;
                                                                                                                                                              
    if ((time(NULL) - lar->sent) >
        strtol(gaim_prefs_get_string("/plugins/gtk/X11/gassist/auto_resend_timeout")
          , NULL, 0)) {
      last_auto_responses = g_slist_remove(last_auto_responses, lar);
      g_free(lar);
    }
  }
                                                                                                                                                              
  return FALSE; /* do not run again */
}


struct last_auto_response *get_last_auto_response(GaimConnection *gc, const char *name)
{
  GSList *tmp;
  struct last_auto_response *lar;
                                                                                                                                                              
  /* because we're modifying or creating a lar, schedule the
   * function to expire them as the pref dictates */
  gaim_timeout_add((strtol(gaim_prefs_get_string("/plugins/gtk/X11/gassist/auto_resend_timeout"),
          NULL, 0) + 1) * 1000, expire_last_auto_responses, NULL);
                                                                                                                                                              
  tmp = last_auto_responses;
                                                                                                                                                              
  while (tmp) {
    lar = (struct last_auto_response *)tmp->data;
                                                                                                                                                              
    if (gc == lar->gc && !strncmp(name, lar->name, sizeof(lar->name)))
      return lar;
                                                                                                                                                              
    tmp = tmp->next;
  }
                                                                                                                                                              
  lar = (struct last_auto_response *)g_new0(struct last_auto_response, 1);
  g_snprintf(lar->name, sizeof(lar->name), "%s", name);
  lar->gc = gc;
  lar->sent = 0;
  last_auto_responses = g_slist_append(last_auto_responses, lar);
                                                                                                                                                              
  return lar;
}
                                                                                                                                                              
void flush_last_auto_responses(GaimConnection *gc)
{
  GSList *tmp, *cur;
  struct last_auto_response *lar;
                                                                                                                                                              
  tmp = last_auto_responses;
                                                                                                                                                              
  while (tmp) {
    cur = tmp;
    tmp = tmp->next;
    lar = (struct last_auto_response *)cur->data;
                                                                                                                                                              
    if (lar->gc == gc) {
      last_auto_responses = g_slist_remove(last_auto_responses, lar);
      g_free(lar);
    }
  }
}


/* --- END STUFF FROM server.c ----*/

static gboolean
chat_recv_im(GaimAccount *account, char *sender, char *message,
             GaimConversation *conv)
{
	// XXX: Dunno if I am going to use this, but it's here
  	// incase I decide to do so.
	return FALSE;
}


static GList *
actions(GaimPlugin *plugin, gpointer context)
{
  GList *l = NULL;
  GaimPluginAction *act = NULL;
  if(gaim_prefs_get_bool("/plugins/gtk/X11/gassist/forward_msgs"))
	  act = gaim_plugin_action_new(_("Disable Gaim-Assistant Forwarding"),
	      toggle_forwarding);
  else
  	act = gaim_plugin_action_new(_("Enable Gaim-Assistant Forwarding"),
	      toggle_forwarding);
  l = g_list_append(l, act);
                                                                                                                                                                                 
  return l;
}

static gboolean
plugin_load(GaimPlugin *plugin)
{

  // load the plugin, register our signals 
  // with all the conversations.
  	init_plugin(plugin);
	void *conv_handle = gaim_conversations_get_handle();

	my_plugin = plugin;

	gaim_signal_connect(conv_handle, "received-im-msg", plugin,
	                    GAIM_CALLBACK(im_recv_im), NULL);
	gaim_signal_connect(conv_handle, "received-chat-msg", plugin,
	                    GAIM_CALLBACK(chat_recv_im), NULL);
	gaim_signal_connect(gaim_blist_get_handle(), "buddy-signed-on", plugin,
                      GAIM_CALLBACK(gasst_buddy_signed_on), NULL);
	setting_from_toggle=0;                      
	return TRUE;
}



static gboolean
plugin_unload(GaimPlugin *plugin)
{
  // free pending queued messages if any exist.
  if(gasst_queue!=NULL){
  	GSList *q = gasst_queue;
  	struct queued_message *qm;
  	while (q) {
  		qm = q->data;
    	g_free(qm->message);
  		g_free(qm);
  
  		q->data = NULL;
	  	q = q->next;
  	}

	  g_slist_free(gasst_queue);
  	gasst_queue = NULL;
  }
	return TRUE;
}




static void init_plugin(GaimPlugin *plugin)
{
gaim_debug(GAIM_DEBUG_INFO, "gaim-assistant", "In init_plugin\n");
  // a list of all our preferences.

  gaim_prefs_add_none("/plugins/gtk");
  gaim_prefs_add_none("/plugins/gtk/X11");
  gaim_prefs_add_none("/plugins/gtk/X11/gassist");

  /* bool prefs */
  gaim_prefs_add_bool("/plugins/gtk/X11/gassist/forward_msgs", FALSE);
  gaim_prefs_add_bool("/plugins/gtk/X11/gassist/que_fwd_msgs", FALSE);
  gaim_prefs_add_bool("/plugins/gtk/X11/gassist/blist_only", FALSE);
  gaim_prefs_add_bool("/plugins/gtk/X11/gassist/show_fwdscrnm", FALSE);
  gaim_prefs_add_bool("/plugins/gtk/X11/gassist/show_fwdscrnm_blist_only", FALSE);

  gaim_prefs_add_bool("/plugins/gtk/X11/gassist/online_check", TRUE);
  gaim_prefs_add_bool("/plugins/gtk/X11/gassist/auto_reply", TRUE);
 
  /* string prefs */
  gaim_prefs_add_string("/plugins/gtk/X11/gassist/fwd_scrnm", "");
  gaim_prefs_add_string("/plugins/gtk/X11/gassist/auto_resend_timeout", "300");
  gaim_prefs_add_string("/plugins/gtk/X11/gassist/origin_account", "");
  
  if(gaim_prefs_exists("/plugins/gtk/X11/gassist/orgin_account")){
  	gaim_debug(GAIM_DEBUG_INFO, "gaim_assistant", "Old origin account pref found. Renaming\n");
	gaim_prefs_rename("/plugins/gtk/X11/gassist/orgin_account",
		"/plugins/gtk/X11/gassist/origin_account");
  }

  // FIXME: remove when que_fwd_msgs is fixed.
  if(gaim_prefs_get_bool("/plugins/gtk/X11/gassist/que_fwd_msgs")){
    gaim_prefs_set_bool("/plugins/gtk/X11/gassist/que_fwd_msgs", FALSE);
    gaim_debug(GAIM_DEBUG_INFO, "gaim_assistant", "Disabling que_fwd_msgs util it is fixed.\n");
  }

}




GAIM_INIT_PLUGIN(gassist, init_plugin, info)
